@@ -1,8 +1,9 @@
+import json
 import sys
 
 
 def wrap_val(v):
-    return "'{}'".format(v) if isinstance(v, str) else v
+    return f"'{v}'" if isinstance(v, str) else v
 
 
 def _is_dunder(name):
@@ -20,6 +21,12 @@ def _is_sunder(name):
     return len(name) > 2 and name[0] == "_" and name[1:2] != "_"
 
 
+def raise_errs_if_needed(errors):
+    if errors:
+        messages = json.dumps([str(e) for e in errors])
+        raise errors[0].__class__(messages) from errors[0]
+
+
 py_version = sys.version_info[0:2]
 python_ver_36 = py_version == (3, 6)
 python_ver_atleast_than_37 = py_version > (3, 6)
