@@ -5,6 +5,7 @@ from collections.abc import Mapping
 from functools import reduce
 from typing import Dict
 
+from .commons import raise_errs_if_needed
 from .versioned_mapping import VERSION_MAPPING, Versioned, convert_dict
 from .mappers import (
     aggregate_deserialization_mappers,
@@ -42,7 +43,6 @@ from .fields import (
     SerializableField,
     SizedCollection,
     wrap_val,
-    Function,
     _DictStruct,
     _ListStruct,
     Deque,
@@ -64,16 +64,14 @@ def deserialize_list_like(
     camel_case_convert=False,
 ):
     if not isinstance(value, (list, tuple, set)):
-        raise ValueError(
-            "{}: Got {}; Expected a list, set, or tuple".format(name, value)
-        )
+        raise ValueError(f"{name}: Got {value}; Expected a list, set, or tuple")
 
     values = []
     items = field.items
     if isinstance(items, Field):
         ignore_none = getattr(items, IGNORE_NONE_VALUES, False)
         for i, v in enumerate(value):
-            item_name = "{}_{}".format(name, i)
+            item_name = f"{name}_{i}"
             try:
                 list_item = deserialize_single_field(
                     items,
@@ -85,10 +83,8 @@ def deserialize_list_like(
                     ignore_none=ignore_none,
                 )
             except (ValueError, TypeError) as e:
-                prefix = (
-                    "" if str(e).startswith(item_name) else "{}: ".format(item_name)
-                )
-                raise ValueError("{}{}".format(prefix, str(e))) from e
+                prefix = "" if str(e).startswith(item_name) else f"{item_name}: "
+                raise ValueError(f"{prefix}{str(e)}") from e
             values.append(list_item)
     elif isinstance(items, (list, tuple)):
         for i, item in enumerate(items):
@@ -104,7 +100,7 @@ def deserialize_list_like(
                     ignore_none=ignore_none,
                 )
             except (ValueError, TypeError) as e:
-                raise ValueError("{}_{}: {}".format(name, i, str(e))) from e
+                raise ValueError(f"{name}_{i}: {str(e)}") from e
             values.append(res)
         values += value[len(items) :]
     else:
@@ -202,44 +198,32 @@ def deserialize_multifield_wrapper(
                 return deserialized
             elif isinstance(field, NotField):
                 raise ValueError(
-                    "{}: Got {}; Matches field {}, but must not match it".format(
-                        name, wrap_val(source_val), field
-                    )
+                    f"{name}: Got {wrap_val(source_val)}; Matches field {field}, but must not match it"
                 )
             elif isinstance(field, OneOf) and found_previous_match:
                 raise ValueError(
-                    "{}: Got {}; Matches more than one match".format(
-                        name, wrap_val(source_val)
-                    )
+                    f"{name}: Got {wrap_val(source_val)}; Matches more than one match"
                 )
             found_previous_match = True
         except Exception as e:
             failures += 1
             err_messages.append(
-                "({}) Does not match {}. reason: {}".format(
-                    len(err_messages) + 1, field_option, str(e)
-                )
+                f"({len(err_messages) + 1}) Does not match {field_option}. reason: {str(e)}"
             )
             if isinstance(field, AllOf):
                 raise ValueError(
-                    "{}: Got {}; Does not match {}. reason: {}".format(
-                        name, wrap_val(source_val), field_option, str(e)
-                    )
+                    f"{name}: Got {wrap_val(source_val)}; Does not match {field_option}. reason: {str(e)}"
                 ) from e
     if failures == len(field.get_fields()) and not isinstance(field, NotField):
         raise ValueError(
-            "{}: Got {}; Does not match any field option: {}".format(
-                name, wrap_val(source_val), ". ".join(err_messages)
-            )
+            f"{name}: Got {wrap_val(source_val)}; Does not match any field option: {'. '.join(err_messages)}"
         )
     return deserialized
 
 
 def deserialize_map(map_field, source_val, name, camel_case_convert=False):
     if not isinstance(source_val, dict):
-        raise TypeError(
-            "{}: Got {}; Expected a dictionary".format(name, wrap_val(source_val))
-        )
+        raise TypeError(f"{name}: Got {wrap_val(source_val)}; Expected a dictionary")
     if map_field.items:
         key_field, value_field = map_field.items
     else:
@@ -351,9 +335,7 @@ def deserialize_single_field(  # pylint: disable=too-many-branches
                 camel_case_convert=camel_case_convert,
             )
         except Exception as e:
-            raise ValueError(
-                "{}: Got {}; {}".format(name, wrap_val(source_val), str(e))
-            ) from e
+            raise ValueError(f"{name}: Got {wrap_val(source_val)}; {str(e)}") from e
     elif isinstance(field, Map):
         value = deserialize_map(
             field, source_val, name, camel_case_convert=camel_case_convert
@@ -369,12 +351,11 @@ def deserialize_single_field(  # pylint: disable=too-many-branches
         elif isinstance(source_val, dict):
             value = ty(**source_val)
     elif isinstance(field, NoneField):
-        raise ValueError("{}: Got {}; Expected None".format(name, wrap_val(source_val)))
+        raise ValueError(f"{name}: Got {wrap_val(source_val)}; Expected None")
     else:
         raise NotImplementedError(
-            "{}: Got {}; Cannot deserialize value of type {}. Are you using non-Typepy class?".format(
-                name, wrap_val(source_val), field.__class__.__name__
-            )
+            f"{name}: Got {wrap_val(source_val)}; Cannot deserialize value of type {field.__class__.__name__}. Are "
+            "you using non-Typepy class? "
         )
     return value
 
@@ -450,10 +431,8 @@ def construct_fields_map(
                     )
                 except (TypeError, ValueError) as ex:
                     errors.append(ex)
-    if errors:
-        messages = json.dumps([str(e) for e in errors])
-        raise errors[0].__class__(messages) from errors[0]
 
+    raise_errs_if_needed(errors)
     return result
 
 
@@ -528,9 +507,7 @@ def deserialize_structure_internal(
                     ignore_none=ignore_none,
                 )
             )
-        raise TypeError(
-            "{}: Expected a dictionary; Got {}".format(name, wrap_val(input_dict))
-        )
+        raise TypeError(f"{name}: Expected a dictionary; Got { wrap_val(input_dict)}")
 
     kwargs = {
         k: v for k, v in input_dict.items() if k not in field_by_name and keep_undefined
@@ -609,9 +586,7 @@ def get_processed_input(key, mapper, the_dict):
         processed_input = val if val is not None else the_dict.get(key)
     else:
         raise TypeError(
-            "mapper value must be a key in the input or a FunctionCal. Got {}".format(
-                wrap_val(key_mapper)
-            )
+            f"mapper value must be a key in the input or a FunctionCal. Got {wrap_val(key_mapper)}"
         )
     return processed_input
 
@@ -625,9 +600,9 @@ def serialize_multifield_wrapper(fields, name, val, mapper, camel_case_convert):
             if isinstance(field, TypedField) and not isinstance(val, field._ty):
                 raise TypeError
             return serialize_field(field, val, camel_case_convert)
-        except:
+        except:  # pylint: disable=bare-except
             pass
-    else:
+    else:  # pylint: disable=useless-else-on-loop
         raise ValueError(f"{name}: cannot serialize value: {val}")
 
 
@@ -895,8 +870,8 @@ def serialize(value, *, mapper: Dict = None, compact=False, camel_case_convert=F
         if isinstance(value, (enum.Enum,)):
             return value.name
         raise TypeError(
-            "serialize: Not a Structure or Field that with an obvious serialization. Got: {}."
-            " Maybe try serialize_field() instead?".format(value)
+            f"serialize: Not a Structure or Field that with an obvious serialization. Got: {value}."
+            " Maybe try serialize_field() instead?"
         )
     return serialize_internal(
         value, mapper=mapper, compact=compact, camel_case_convert=camel_case_convert
